local utils = require "utils"

local function matomo_tracking_code(site_id)
    return [=[
<!-- Matomo -->
<script>
  var _paq = window._paq = window._paq || [];
  var p = "https://REWRITE_JASIMA_HOST_PLACEHOLDER" + window.location.pathname;
  _paq.push(["setCustomUrl", p]);
  _paq.push(["setExcludedQueryParams", ["roomName","account","accountnum","address","address1","address2","address3","addressline1","addressline2","adres","adresse","age","alter","auth","authpw","bic","billingaddress","billingaddress1","billingaddress2","calle","cardnumber","cc","ccc","cccsc","cccvc","cccvv","ccexpiry","ccexpmonth","ccexpyear","ccname","ccnumber","cctype","cell","cellphone","city","clientid","clientsecret","company","consumerkey","consumersecret","contrasenya","contrase\u00f1a","creditcard","creditcardnumber","cvc","cvv","dateofbirth","debitcard","direcci\u00f3n","dob","domain","ebost","email","emailaddress","emailadresse","epos","epost","eposta","exp","familyname","firma","firstname","formlogin","fullname","gender","geschlecht","gst","gstnumber","handynummer","has\u0142o","heslo","iban","ibanaccountnum","ibanaccountnumber","id","identifier","indirizzo","kartakredytowa","kennwort","keyconsumerkey","keyconsumersecret","konto","kontonr","kontonummer","kredietkaart","kreditkarte","kreditkort","lastname","login","mail","mobiili","mobile","mobilne","nachname","name","nickname","false","osoite","parole","pass","passord","password","passwort","pasword","paswort","paword","phone","pin","plz","postalcode","postcode","postleitzahl","privatekey","publickey","pw","pwd","pword","pwrd","rue","secret","secretq","secretquestion","shippingaddress","shippingaddress1","shippingaddress2","socialsec","socialsecuritynumber","socsec","sokak","ssn","steuernummer","strasse","street","surname","swift","tax","taxnumber","tel","telefon","telefonnr","telefonnummer","telefono","telephone","token","token_auth","tokenauth","t\u00e9l\u00e9phone","ulica","user","username","vat","vatnumber","via","vorname","wachtwoord","wagwoord","webhooksecret","website","zip","zipcode"]]);
  _paq.push(["trackPageView", p]);
  _paq.push(["enableLinkTracking"]);
  (function() {
    var u="//]=] .. os.getenv("JASIMA_MATOMO_HOST") .. [=[/";
    _paq.push(["setTrackerUrl", u+"matomo.php"]);
    _paq.push(["setSiteId", "]=] .. tostring(ngx.ctx.jasima_config.matomo_site_id) .. [=["]);
    var d=document, g=d.createElement("script"), s=d.getElementsByTagName("script")[0];
    g.async=true; g.src=u+"matomo.js"; s.parentNode.insertBefore(g,s);
  })();
</script>
<!-- End Matomo Code -->
</body>
    ]=]
end

local function rewrite_body(body)
    -- Rewrite links for assets and outbound links to other mirrored sites
    local pool_map = ngx.ctx.jasima_pool_map
    if pool_map then
        body = body:gsub("//([a-zA-Z0-9%.%-]+%.[a-zA-Z0-9]+)/", utils.get_mirror)
    end
    -- Add Matomo tracking code
    if ngx.ctx.jasima_config.matomo_site_id then
        body = body:gsub("</body>", matomo_tracking_code(ngx.ctx.jasima_config.matomo_site_id))
        -- TODO: Ensure that tracking code was added when it's HTML, but only for HTML
    end
    -- Handle first party Tealium installations
    if ngx.ctx.jasima_host_tealium then
        local escaped_host = ngx.ctx.jasima_host_tealium:gsub("%.", "%%.")
        body = body:gsub("(https:)??//" .. escaped_host .. "/", "/utag/" .. ngx.ctx.jasima_config.first_party_tealium.account .. "/")
        body = body:gsub("//tags.tiqcdn.com/", "/")
        if ngx.var.jasima_host_connect == "tags.tiqcdn.com" and ngx.ctx.jasima_host_adobe then
            body, count = body:gsub([[return"http"%+%(a%.ssl%?"s":""%)%+"://"%+b%+"/b/ss/]], [[return"/b/ss/]])
            ngx.log(ngx.DEBUG, "Performing rewrite for Adobe Analytics in Tealium tag " .. count)
        end
    end
    -- Handle first party Adobe Analytics
    if ngx.ctx.jasima_host_adobe then
        local escaped_host = ngx.ctx.jasima_host_adobe:gsub("%.", "%%.")
        body = body:gsub("(https:)??//" .. escaped_host .. "/", "/")
    end
    return body
end

if ngx.ctx.rewriting then
    local chunk = ngx.arg[1]
    local eof = ngx.arg[2]

    ngx.ctx.buffered = (ngx.ctx.buffered or "") .. (chunk or "")

    if #ngx.ctx.buffered > 5 * 1024 * 1024 and not eof then
        -- Don't just consume memory forever
        ngx.arg[1] = rewrite_body(ngx.ctx.buffered)  -- We still do our best
        ngx.ctx.rewriting = false
        return
    end

    if eof then
        ngx.arg[1] = rewrite_body(ngx.ctx.buffered)
    else
        ngx.arg[1] = nil
    end
end
